C$Procedure      GETFOV (Instrument FOV configuration)
 
      SUBROUTINE GETFOV( INSTID, ROOM, SHAPE, FRAME, BSIGHT, N, BOUNDS)
 
C$ Abstract
C
C     This subroutine returns the FOV configuration for a specified
C     instrument.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     None.
C
C$ Declarations
 
      IMPLICIT NONE
 
      INTEGER               INSTID
      INTEGER               ROOM
      CHARACTER*(*)         SHAPE
      CHARACTER*(*)         FRAME
      DOUBLE PRECISION      BSIGHT (3)
      INTEGER               N
      DOUBLE PRECISION      BOUNDS ( 3, *)
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     INSTID     I   NAIF ID of an instrument
C     ROOM       I   Maximum number of vectors that can be returned.
C     SHAPE      O   Instrument FOV shape.
C     FRAME      O   Name of the frame FOV vectors defined in.
C     BSIGHT     O   Boresight vector.
C     N          O   Number of boundary vectors returned.
C     BOUNDS     O   FOV boundary vectors.
C
C$ Detailed_Input
C
C     INSTID     NAIF ID of an instrument.
C
C     ROOM       The amount of room allocated in BOUNDS for returning
C                components of boundary vectors.
C
C$ Detailed_Output
C
C     SHAPE      is a character string that describes the "shape" of
C                the field of view.  Possible values returned are:
C
C                   'POLYGON',
C                   'CIRCLE',
C                   'ELLIPSE'
C
C                If the value of SHAPE is 'POLYGON' the field of
C                view of the instrument is a pyramidal polyhedra.
C                The vertex of the pyramid is at the instrument and
C                the rays along the edges of the pyramid are parallel
C                to the vectors returned in BOUNDS.
C
C                If the value of SHAPE is 'CIRCLE' the field of
C                view of the instrument is a circular cone about the
C                boresite vector.  A single vector will be returned
C                in BOUNDS.  This vector will be parallel to a ray
C                that lies in the cone that makes up the boundary of
C                the field of view.
C
C                If the value of SHAPE is 'ELLIPSE' the field of
C                view of the instrument is a elliptical cone with the
C                boresite vector as the axis of the cone.  In this case
C                two vectors are returned in BOUNDS. One of the
C                vectors returned in BOUNDS points to the end of the
C                semi-major axis of a perpendicular cross section of
C                the elliptic cone.  The other vector points to the
C                end of the semi-minor axis  of a perpendicular
C                cross section of the cone.
C
C
C     FRAME      is the name of the frame in which the field of
C                view boundary vectors are defined.
C
C     BSIGHT     is a vector that points in the direction of the
C                center of the field of view.  The length of BSIGHT
C                is not specified other than being non-zero.
C
C     N          is the number of boundary vectors returned.
C
C     BOUNDS     is an array of vectors that point to the "corners"
C                of the instrument field of view.  (See the discussion
C                accompanying shape for an expansion of the term
C                "corner of the field of view."  Note that the vectors
C                returned in BOUNDS are not necessarily unit vectors.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     This routine relies upon having succsessfully loaded an
C     instrument kernel (IK-file) via the routine LDPOOL prior
C     to calling this routine.
C
C$ Exceptions
C
C
C     1) The error 'SPICE(FRAMEKWDNOTFOUND)' is signalled if
C        the frame associated with the instrument can not be found
C        in the kernel pool.
C
C     2) The error 'SPICE(SHAPEKWDNOTFOUND)' is signalled if
C        the shape of the instrument field of view can not be
C        found in the kernel pool.
C
C     3) The error 'SPICE(SHAPENOTSUPPORTED)' is signalled if
C        the shape specified by the instrument kernel is not one
C        of the three values: 'CIRCLE', 'POLYGON', 'ELLIPSE'.
C
C     4) The error 'SPICE(BORESIGHTKWDNOTFOUND)' is signalled if
C        the direction of the boresight cannot be located in the
C        kernel pool.
C
C     5) The error 'SPICE(BADBORESIGHTSPEC)' is signalled if
C        the number of components for the boresight vector
C        in the kernel pool is not 3.
C
C     6) The error 'SPICE(BOUNDARYKWDNOTFOUND)' is signalled if
C        the boundary vectors for the edge of the field of view
C        cannot be found in the kernel pool.
C
C     7) The error 'SPICE(TOOMANYBOUNDARYVECTORS)' is signalled if
C        there is insufficient room (as specified by the variable ROOM)
C        to return all of the vectors associated with the boundary
C        of the field of view.
C
C     8) The error 'SPICE(BADBOUNDARYDEFINITION)' is signalled if
C        the number of components of vectors making up the field
C        of view is not a multiple of 3.
C
C$ Particulars
C
C     This routine provides a common interface to retrieving
C     the geometric characteristics of an instrument field of
C     view for a wide variety of remote sensing instruments
C     across many different space missions.
C
C     Given the NAIF instrument ID, (and having "loaded" the
C     instrument field of view description via the routine LDPOOL)
C     this routine returns the bore-sight of the instrument, the
C     "shape" of the field of view, a collection of vectors
C     that point along the edges of the field of view, and the
C     name of the reference frame in which these vectors are defined.
C
C     This routine is intended to be an intermediate level routine.
C     It is expected that users of this routine will be familiar
C     with the SPICE frames subsystem and will be comfortable writing
C     software to further manipulate the vectors retrieved by this
C     routine.
C
C$ Examples
C
C     Suppose you need to determine where the planetocentric
C     latitude and longitude of the points
C     where the corners of a rectangular field of view intersect
C     the surface of an object.  This routine together with the
C     a handful of other SPICE routine allows you to easily make
C     this determinations.
C
C    (Note that this example assumes that all necessary SPICE kernels
C     have been loaded.)
C
C
C        DOUBLE PRECISION      BOUNDS( 3, 4 )
C        DOUBLE PRECISION      BSITE ( 3 )
C        DOUBLE PRECISION      STATE ( 6 )
C        DOUBLE PRECISION      POSOBS( 3 )
C
C        INTEGER               BODY
C        INTEGER               INSTID
C
C
C        Determine the object, instrument and time
C        at which the latitude and longitude should be computed.
C
C        IMPLICIT NONE
C
C        SPICELIB Functions
C
C        DOUBLE PRECISION      DPR
C
C
C        Local Variables and Parameters
C
C        INTEGER               WDSIZE
C        PARAMETER           ( WDSIZE = 32 )
C
C        CHARACTER*(WDSIZE)    BDYFXD
C        CHARACTER*(WDSIZE)    FRAME
C        CHARACTER*(WDSIZE)    SHAPE
C
C        DOUBLE PRECISION      ABC    ( 3 )
C        DOUBLE PRECISION      BOUNDS ( 3, 4 )
C        DOUBLE PRECISION      BSIGHT ( 3 )
C        DOUBLE PRECISION      ET
C        DOUBLE PRECISION      FOV    ( 3, 4 )
C        DOUBLE PRECISION      LAT
C        DOUBLE PRECISION      LONG
C        DOUBLE PRECISION      LT
C        DOUBLE PRECISION      POINT  ( 3 )
C        DOUBLE PRECISION      POSOBS ( 3 )
C        DOUBLE PRECISION      RADIUS
C        DOUBLE PRECISION      ROT    ( 3, 3 )
C        DOUBLE PRECISION      STATE  ( 6 )
C        DOUBLE PRECISION      FRM2J2 ( 6, 6 )
C        DOUBLE PRECISION      TSIPM  ( 6, 6 )
C        DOUBLE PRECISION      XFORM  ( 6, 6 )
C
C        INTEGER               BODY
C        INTEGER               DIM
C        INTEGER               FRCODE
C        INTEGER               I
C        INTEGER               INSTID
C        INTEGER               J
C        INTEGER               N
C        INTEGER               OBS
C        INTEGER               ROOM
C
C        LOGICAL               FOUND
C
C
C        CALL GET_PARTICULARS ( INSTID, BODY, ET )
C
C        Get the field of view information
C
C        CALL GETFOV ( INSTID, ROOM, SHAPE, FRAME, BSIGHT, N, BOUNDS )
C
C
C        We use the normal convention:  The observer can be obtained
C        by performing an integer divide on the instrument ID.
C
C        OBS = INSTID / 1000
C
C        Determine the bodyfixed frame associated with the body
C        being observed.
C
C        CALL CIDFRM ( BODY, FRCODE, BDYFXD, FOUND )
C
C        IF ( .NOT. FOUND ) THEN
C           WRITE (*,*) 'Could not get the bodyfixed frame for : ', BODY
C           STOP
C        END IF
C
C        Get the state of the body relative to the bodyfixed frame
C        corrected for light time.  Then compute the apparent position
C        of the observer in the bodyfixed frame.
C
C        CALL SPKEZ ( BODY, ET, BDYFXD, 'LT+S', OBS, STATE, LT )
C        CALL VMINUS( STATE, POSOBS )
C
C        Get the transformation from the instrument frame to the
C        bodyfixed frame of the body.
C
C        CALL SXFORM ( FRAME,  'J2000', ET,    FRM2J2 )
C        CALL SXFORM ( 'J2000', BDYFXD, ET-LT, TSIPM  )
C        CALL MXMG   ( TSIPM,   FRM2J2, 6, 6,  XFORM  )
C
C        We only need the rotation part of the state transformation
C        matrix.
C
C        DO I = 1,3
C           DO J = 1,3
C              ROT(I,J) = XFORM(I,J)
C           END DO
C        END DO
C
C        Rotate the field of view into the bodyfixed frame.
C
C        DO I = 1, N
C           CALL MXV ( ROT, BOUNDS(1,I), FOV(1,I) )
C        END DO
C
C        Look up the axes of the body.
C
C        CALL BODVAR ( BODY, 'RADII', DIM, ABC )
C
C        We assume that all of the edges of the field of view
C        intersect the body.  A more robust example would check
C        the FOUND flag after calling SURFPT.
C
C        WRITE (*,*) 'The latitude and longitude of the corners are: '
C
C        DO I = 1, N
C           CALL SURFPT ( POSOBS, FOV(1,I), ABC(1), ABC(2), ABC(3),
C       .                 POINT,  FOUND )
C           CALL RECLAT ( POINT, LAT, LONG, RADIUS )
C
C           WRITE (*,*)
C           WRITE (*,*) '    Corner        :', I
C           WRITE (*,*) '    Latitude (deg):', LAT*DPR()
C           WRITE (*,*) '    Longitude(deg):', LONG*DPR()
C        END DO
C
C
C$ Restrictions
C
C     An I-kernel for the instrument specified in INSTID must have
C     been loaded via a call to LDPOOL prior to calling this routine.
C     Furthermore, the specification for the instrument field of
C     view must be consistent with the expectations of this routine.
C     These expectations are outlined in the document IKERNELS.
C
C$ Author_and_Institution
C
C     B.V. Semenov (JPL)
C     W.L. Taber   (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    Beta Version 2.2.0, 01-FEB-1999    (JAB)
C
C        Deleted an unnecessary declaration.  Updated
C        copyright year.
C
C-    Beta Version 2.1.0, 03-SEP-1998    (WLT)
C
C        The code example above was corrected.  Previously the
C        Apparent intercept directions were not properly transformed
C        into the light time corrected bodyfixed frame.  Thanks to
C        Irina Mikheeva of LPL at the University of Arizona for
C        bringing this error to the attention of NAIF.
C
C-    Beta Version 2.0.0, 18-JUN-1997    (WLT)
C
C        The meaning of the arguments ROOM and N were modified
C        so as to refer to the number of vectors and not the number
C        of vector components.  In addition the array BUNDRY was
C        renames BOUNDS and redeclared to be an array of 3-vectors.
C
C-    Beta Version 1.0.0, April 17, 1997 (BVS)
C
C
C-&
 
C$ Index_Entries
C
C     return instrument's FOV configuration
C
C-&
 
C
C     Local parameters
C
 
      INTEGER               KWDNLN
      PARAMETER           ( KWDNLN = 32 )
 
      INTEGER               NSHAPS
      PARAMETER           ( NSHAPS = 3  )
 
      CHARACTER*(*)         KWFRAM
      PARAMETER           ( KWFRAM = 'INS#_FOV_FRAME' )
 
      CHARACTER*(*)         KWSHAP
      PARAMETER           ( KWSHAP = 'INS#_FOV_SHAPE' )
 
      CHARACTER*(*)         KWBORE
      PARAMETER           ( KWBORE = 'INS#_BORESIGHT' )
 
      CHARACTER*(*)         KWBOUN
      PARAMETER           ( KWBOUN = 'INS#_FOV_BOUNDARY_CORNERS' )
 
C
C     Local variables
C
 
      CHARACTER*(KWDNLN)    KWORD
 
      CHARACTER*(KWDNLN)    SHAPID ( NSHAPS )
 
      LOGICAL               FOUND
      CHARACTER*(1)         TYPE
 
      INTEGER               I
      INTEGER               MXCMP
 
C
C     SPICELIB functions
C
      INTEGER               RTRIM
      INTEGER               BSRCHC
      LOGICAL               RETURN
 
C
C     Allowed values of shape identifier.
C
      DATA  ( SHAPID (I), I =  1, NSHAPS )
     .     /
     .     'CIRCLE',
     .     'ELLIPSE',
     .     'POLYGON'
     .     /
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'GETFOV' )
      END IF
 
      MXCMP = 3*ROOM
C
C     Look for the frame keyword and get frame name if found,
C     complain if not.
C
      CALL REPMI  ( KWFRAM, '#', INSTID, KWORD )
      CALL GCPOOL ( KWORD, 1, 1, I, FRAME, FOUND )
 
      IF ( .NOT. FOUND ) THEN
 
         CALL SETMSG ( 'The variable, ''#'', specifying the frame '
     .   //            'which instrument # FOV components are '
     .   //            'defined relative to was not found in the '
     .   //            'kernel pool. Check whether IK file for '
     .   //            'the instrument was loaded into the program '
     .   //            'and whether this variable is specified in '
     .   //            'that file.' )
         CALL ERRCH  ( '#', KWORD(1:RTRIM(KWORD)) )
         CALL ERRINT ( '#', INSTID               )
         CALL SIGERR ( 'SPICE(FRAMEKWDNOTFOUND)' )
         CALL CHKOUT ( 'GETFOV'                  )
         RETURN
 
      END IF
 
C
C     Look for the shape keyword and get shape identifier if found,
C     complain if not.
C
      CALL REPMI  ( KWSHAP, '#', INSTID, KWORD )
      CALL GCPOOL ( KWORD, 1, 1, I, SHAPE, FOUND )
 
      IF ( .NOT. FOUND ) THEN
 
         CALL SETMSG ( 'The variable, ''#'', specifying the shape '
     .   //            'of the instrument # FOV was not found in the '
     .   //            'kernel pool. Check whether IK file for '
     .   //            'the instrument was loaded into the program '
     .   //            'and whether this variable is specified in '
     .   //            'that file.' )
         CALL ERRCH  ( '#', KWORD(1:RTRIM(KWORD)) )
         CALL ERRINT ( '#', INSTID                )
         CALL SIGERR ( 'SPICE(SHAPEKWDNOTFOUND)'  )
         CALL CHKOUT ( 'GETFOV'                   )
         RETURN
 
      END IF
 
C
C     Check whether shape identified that we got is one from the list
C     of supported, complain if not.
C
      IF ( BSRCHC( SHAPE(1:RTRIM(SHAPE)), NSHAPS, SHAPID ) .EQ. 0) THEN
 
         CALL SETMSG ( 'The FOV shape, ''#'', specified in the '
     .   //            'keyword, ''#'', for the instrument # '
     .   //            'is not supported. See GETFOV subroutine '
     .   //            'header for the list of supported instrument '
     .   //            'FOV shapes.' )
         CALL ERRCH  ( '#', SHAPE(1:RTRIM(SHAPE)) )
         CALL ERRCH  ( '#', KWORD(1:RTRIM(KWORD)) )
         CALL ERRINT ( '#', INSTID                )
         CALL SIGERR ( 'SPICE(SHAPENOTSUPPORTED)' )
         CALL CHKOUT ( 'GETFOV'                   )
         RETURN
 
      END IF
 
C
C     Look for the boresight keyword and get boresight vector if found,
C     complain if not.
C
      CALL REPMI  ( KWBORE, '#', INSTID, KWORD )
      CALL GDPOOL ( KWORD, 1, 3, I, BSIGHT, FOUND )
 
      IF ( .NOT. FOUND ) THEN
 
         CALL SETMSG ( 'The variable, ''#'', specifying the boresight '
     .   //            'of the instrument # was not found in the '
     .   //            'kernel pool. Check whether IK file for '
     .   //            'the instrument was loaded into the program '
     .   //            'and whether this variable is specified in '
     .   //            'that file.' )
         CALL ERRCH  ( '#', KWORD(1:RTRIM(KWORD))    )
         CALL ERRINT ( '#', INSTID                   )
         CALL SIGERR ( 'SPICE(BORESIGHTKWDNOTFOUND)' )
         CALL CHKOUT ( 'GETFOV'                      )
         RETURN
 
      END IF
 
C
C     Check whether boresight specified by three coordinates;
C     complain if not.
C
      IF ( I .NE. 3 ) THEN
 
         CALL SETMSG ( 'The number of the boresight vector components '
     .   //            'specified in the ''#'' variable is less than '
     .   //            '3. Correct it in the corresponding IK file'
     .   //            'to be a 3-dimensional vector.')
         CALL ERRCH  ( '#', KWORD(1:RTRIM(KWORD))    )
         CALL SIGERR ( 'SPICE(BADBORESIGHTSPEC)'     )
         CALL CHKOUT ( 'GETFOV'                      )
         RETURN
 
      END IF
 
C
C     Look for the FOV boundary vectors, check whether output array
C     is big enough to hold them; complain if not.
C
      CALL REPMI  ( KWBOUN, '#', INSTID, KWORD )
      CALL DTPOOL ( KWORD, FOUND, N, TYPE )
 
      IF ( .NOT. FOUND ) THEN
 
         CALL SETMSG ( 'The variable, ''#'', specifying the boundary '
     .   //            'vectors of the instrument # FOV '
     .   //            'was not found in the '
     .   //            'kernel pool. Check whether IK file for '
     .   //            'the instrument was loaded into the program '
     .   //            'and whether this variable is specified in '
     .   //            'that file.' )
         CALL ERRCH  ( '#', KWORD(1:RTRIM(KWORD))    )
         CALL ERRINT ( '#', INSTID                   )
         CALL SIGERR ( 'SPICE(BOUNDARYKWDNOTFOUND)'  )
         CALL CHKOUT ( 'GETFOV'                      )
         RETURN
 
      END IF
 
 
C
C     Check whether we have enough room to get all boundary vectors,
C     complain if not.
C
      IF ( N .GT. MXCMP ) THEN
 
         CALL SETMSG ( 'The number of boundary vector components '
     .   //            'specified in the ''#'' pool variable is '
     .   //            'bigger than room to hold them in output array '
     .   //            'specified by the ROOM input variable of the '
     .   //            'GETFOV subroutine.' )
         CALL ERRCH  ( '#', KWORD(1:RTRIM(KWORD))      )
         CALL SIGERR ( 'SPICE(TOOMANYBOUNDARYVECTORS)' )
         CALL CHKOUT ( 'GETFOV'                        )
         RETURN
 
      END IF
 
C
C     Check whether number of boundary components can be divided by 3
C     without reminder.
C
      IF ( MOD( N, 3 ) .NE. 0 ) THEN
 
         CALL SETMSG ( 'The boundary vector components '
     .   //            'specified in the ''#'' pool variable do  '
     .   //            'not represent a set of 3-dimensional '
     .   //            'vectors. Number of components assigned '
     .   //            'to the variable cannot be divided by 3 '
     .   //            'without reminder. ' )
         CALL ERRCH  ( '#', KWORD(1:RTRIM(KWORD))      )
         CALL SIGERR ( 'SPICE(BADBOUNDARYDEFINITION)'  )
         CALL CHKOUT ( 'GETFOV'                        )
         RETURN
 
      END IF
 
C
C     Boundaries are OK. Get them.
C
      CALL GDPOOL ( KWORD, 1, MXCMP, N, BOUNDS, FOUND )
 
      N = N/3
C
C     All done.
C
 
C
C     Standard SPICE error handling.
C
      CALL CHKOUT ( 'GETFOV' )
      RETURN
 
      END
