C$Procedure      MKMASK ( Set plate mutual visibility flag )
 
      SUBROUTINE MKMASK ( NP,    VERT,  NORM,  CENT,
     .                    V1,    V2,    V3,
     .                    PMASK, MAXELE             )
 
C$ Abstract
C
C     For each plate in the model, this routine determines if other
C     plates can be seen from it.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     plate mask
C
C$ Declarations
 
      IMPLICIT NONE
 
 
      INTEGER               NP
      INTEGER               V1     ( * )
      INTEGER               V2     ( * )
      INTEGER               V3     ( * )
      INTEGER               PMASK  ( * )
 
 
      DOUBLE PRECISION      MAXELE ( * )
      DOUBLE PRECISION      NORM   ( 3, * )
      DOUBLE PRECISION      CENT   ( 3, * )
      DOUBLE PRECISION      VERT   ( 3, 3, * )
 
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NP         I   Number of plates.
C     VERT       I   Array containing plate vertex coordinates.
C     NORM       I   Array containing plate outward normal vectors.
C     CENT       I   Array containing plate center coordinates.
C     V1         I   Array of plate vertex IDs (first vertex).
C     V2         I   Array of plate vertex IDs (second vertex).
C     V3         I   Array of plate vertex IDs (third vertex).
C     PMASK      O   Array of mask flag values.
C     MXELEV     O   Array of maximum occluded elevations.
C
C$ Detailed_Input
C
C     NP         Number of plates to be checked for mutual visibility.
C
C     VERT       Array of vertex coordinates of each plate's vertices.
C
C     NORM       Array of double precision 3-vectors containing each
C                plate's outward-pointing normal vector.
C
C     CENT       Array of double precision 3-vectors containing the
C                location of each plate's center.
C
C$ Detailed_Output
C
C     PMASK      Mask flag values: 0 if no plate is visible and 1
C                if it is.
C
C     MAXELE     Maximum elevation of any obstruction visible from
C                plate-A.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Particulars
C
C     Later.
C
C$ Examples
C
C     Later.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 03-JAN-1999 (JAB)
C
C        Made this routine independent of the plate manager,
C        by passing in vertex ID arrays instead of calling
C        a plate manager routine.
C
C-    SPICELIB Version 1.0.0, 28-JAN-1999 (JAB)
C
C
C-&
 
C$ Index_Entries
C
C     plate mask
C
C-&
 
C
C     Local Variables.
C
 
      LOGICAL               RETURN
      LOGICAL               INPRZM
      LOGICAL               ABOVE     ( 3 )
 
      INTEGER               I
      INTEGER               IA
      INTEGER               IB
      INTEGER               NVIS
      INTEGER               VISLST    ( 3 )
      INTEGER               AVID      ( 3 )
      INTEGER               BVID      ( 3 )
 
      DOUBLE PRECISION      ACENT     ( 3 )
      DOUBLE PRECISION      ANORM     ( 3 )
      DOUBLE PRECISION      AVERT     ( 3, 3 )
      DOUBLE PRECISION      BVERT     ( 3, 3 )
 
 
 
C
C     Standard SPICE error handling.
C
 
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN  ( 'MKMASK' )
 
C
C     Initialize mask arrays.
C
 
      DO I = 1, NP
         PMASK(I)  = 0
         MAXELE(I) = 0.D0
      END DO
 
C
C     Outer loop over all plates.
C
 
      DO IA = 1, NP
 
C
C     Get plate-A's vertex IDs.
C
         AVID(1) = V1( IA )
         AVID(2) = V2( IA )
         AVID(3) = V3( IA )
 
C
C     Inner loop over all other plates.
C
 
         IB = 1
 
         DO WHILE ( (PMASK(IA) .EQ. 0) .AND. (IB .LE. NP) )
 
            IB = IB + 1
 
C
C     Skip case where a plate looks at itself.
C
 
            IF ( IA .NE. IB ) THEN
 
C
C     Get plate-B's vertex IDs.
C
 
               BVID(1) = V1( IB )
               BVID(2) = V2( IB )
               BVID(3) = V3( IB )
 
C
C     Give some vectors shorter names.
C
               CALL VEQU ( VERT(1,1,IA), AVERT )
               CALL VEQU ( VERT(1,1,IB), BVERT(1,1) )
               CALL VEQU ( VERT(1,2,IB), BVERT(1,2) )
               CALL VEQU ( VERT(1,3,IB), BVERT(1,3) )
               CALL VEQU ( NORM(1,IA),   ANORM )
               CALL VEQU ( CENT(1,IA),   ACENT )
 
C*********************************************************************
C     Are any vertices of plate-B above plate-A's "local horizon"?
C     If yes, return a list of them.  If plate-A is connected
C     to plate-B, common vertices are not listed.
C*********************************************************************
 
               CALL TRIPUP ( AVID,   BVID, ACENT,  ANORM,
     .                       BVERT,  NVIS, VISLST, ABOVE  )
 
 
               IF ( NVIS .GT. 0 ) THEN
 
 
C*********************************************************************
C     Does any part of plate-B lie within the right triangular prism
C     having plate-A at one end?  If the answer is yes, set
C     the maximum elevation for plate-A at 90 degrees.  If no,
C     determine the maximum elevation by checking every case.
C*********************************************************************
 
                  IF ( INPRZM ( AVERT, BVERT, ANORM, ACENT ) ) THEN
 
                     MAXELE(IA) = 90.D0
 
                  ELSE
 
                     CALL TRAZEL ( AVERT, ANORM, BVERT,
     .                             ABOVE, MAXELE(IA)    )
                  END IF
 
                  PMASK(IA) = 1
 
               END IF
 
            END IF
 
            IB = IB + 1
 
C
C     End of B-plate loop
C
 
         END DO
 
C
C     End of A-plate loop.
C
 
      END DO
 
 
C         DO I = 1, NP
C            WRITE(*,*) 'plate ID, pmask, maxele = ', I,
C     .                  PMASK(I), MAXELE(I)
C         END DO
 
      CALL CHKOUT ( 'MKMASK' )
 
      RETURN
      END
