C$Procedure   RDFFPL ( read triangular plate model flatfile )
 
      SUBROUTINE RDFFPL ( INFILE, NV, VID, X,  Y,  Z,
     .                            NP, PID, V1, V2, V3 )
 
C$ Abstract
C
C     Read a triangular plate model flatfile's vertex and plate data.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     MKPLAT User's Guide
C     SURFACE
C
C$ Keywords
C
C     PLATE
C     FLATFILE
C
C$ Declarations
 
      IMPLICIT              NONE
 
      INCLUDE               'pltmax.inc'
 
      INTEGER               NV
      INTEGER               NP
      INTEGER               V1    ( * )
      INTEGER               V2    ( * )
      INTEGER               V3    ( * )
      INTEGER               VID   ( * )
      INTEGER               PID   ( * )
 
      CHARACTER*(*)         INFILE
 
      DOUBLE PRECISION      X     ( * )
      DOUBLE PRECISION      Y     ( * )
      DOUBLE PRECISION      Z     ( * )
 
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     INFILE     I   Name of plate model flatfile.
C     NV         O   Number of vertices in model.
C     VID        O   Vertex index array.
C     X          O   Vertex X-coordinate array.
C     Y          O   Vertex Y-coordinate array.
C     Z          O   Vertex Z-coordinate array.
C     NP         O   Number of plates in model.
C     PID        O   Plate index array.
C     V1         O   First plate vertex index array.
C     V2         O   Second plate vertex index array.
C     V3         O   Third plate vertex index array.
C
C
C$ Detailed_Input
C
C     INFILE     is the full pathname of the plate model flatfile.
C
C
C$ Detailed_Output
C
C     NV         is the number of vertices in the plate model.
C
C     VID        is an array of NV vertex ID's.
C
C     X          is the array of X-coordinates of the NV vertices,
C                given in a body-fixed frame of reference.
C
C     Y          is the array of Y-coordinates of the NV vertices,
C                given in a body-fixed frame of reference.
C
C     Z          is the array of Z-coordinates of the NV vertices
C                given in a body-fixed frame of reference.
C
C     NP         is the number of triangular plates in the model.
C
C     PID        is the array of NP plate ID's.
C
C     V1         is the array of first plate vertex ID's.
C
C     V2         is the array of second plate vertex ID's.
C
C     V3         is the array of third plate vertex ID's.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the number of vertices (NV) exceeds the value of MAXVRT,
C        SPICE(TOOMANYVERTICES) is signaled.
C
C     2) If the number of plates (NP) exceeds the value of MAXPLT,
C        SPICE(TOOMANYPLATES) is signaled.
C
C     3) If an unexpected end of file is encountered,
C        SPICE(UNEXPECTEDENDOFFILE) is signaled.
C
C$ Particulars
C
C     Input File Format
C     -----------------
C
C     INFILE is assumed to be structured as follows (values
C         are space delimited within each input record).
C
C     Line  1: NV
C         NV = the number of vertices (INTEGER)
C
C     Lines 2 to (NV+1): VID X Y Z
C         VID = the vertex ID (INTEGER)
C         X, Y, and Z = the vertex coordinates (DOUBLE PRECISION)
C
C     Line  (NV+2): NP
C         NP = number of triangular plates (INTEGER)
C
C     Lines (NV+3) to (NV+3+NP+1): PID V1 V2 V3
C         PID = the plate ID (INTEGER)
C         V1, V2, and V3 = the plate vertex ID's (INTEGER)
C
C
C$ Examples
C
C  C     The following include file defines MAXVRT and
C  C     MAXPLT, the maximum number of vertices and plates
C  C     used in the plate model software.
C
C        INCLUDE               'pltmax.inc'
C
C        INTEGER               NV
C        INTEGER               NP
C        INTEGER               PID     ( MAXPLT )
C        INTEGER               VID     ( MAXVRT )
C        INTEGER               V1      ( MAXVRT )
C        INTEGER               V2      ( MAXVRT )
C        INTEGER               V3      ( MAXVRT )
C
C        DOUBLE PRECISION      X       ( MAXVRT )
C        DOUBLE PRECISION      Y       ( MAXVRT )
C        DOUBLE PRECISION      Z       ( MAXVRT )
C
C        CHARACTER*(80)        INFILE
C
C
C        CALL PROMPT ( 'Enter name of plate model file', INFILE )
C
C        CALL RDFFPL ( INFILE, NV, VID, X,  Y,  Z,
C       .                      NP, PID, V1, V2, V3 )
C
C$ Restrictions
C
C     It is the user's responsibility to properly dimension
C     arrays in the calling routine large enough to accept data
C     from the given file.
C
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 22-OCT-1998 (JAB)
C
C-    SPICELIB Version 1.0.0, 09-APR-1997 (JAB)
C
C-&
 
C$ Index_Entries
C
C     read triangular plate model flatfile
C
C-&
 
C
C     SPICELIB functions
C
 
      LOGICAL               RETURN
 
C
C     Local variables
C
 
      LOGICAL               EOF
 
      INTEGER               I
      INTEGER               ND
      INTEGER               NI
      INTEGER               NREC
      INTEGER               ARI     ( 4 )
 
      CHARACTER*(10)        FORMAT
 
      DOUBLE PRECISION      ARD     ( 3 )
 
C
C     Standard SPICE error handling.
C
 
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'RDFFPL' )
      END IF
  
C
C     Read number of vertices.
C
      NREC   = 1
      FORMAT = 'I'
 
      CALL RDFFDI ( INFILE, NREC, FORMAT, ND, NI, ARD, ARI, EOF  )
      IF ( EOF ) THEN
         CALL EXTMSI ( 'End of file after line #.', '#', NREC )
      END IF
 
      NV = ARI(1)
 
      IF ( NV .GT. MAXVRT ) THEN
         WRITE(*,*) 'NV, MAXVRT = ', NV, MAXVRT
         CALL SETMSG ( 'Number of vertices exceeds limit.'        )
         CALL SIGERR ( 'SPICE(TOOMANYVERTICES)'                   )
      END IF
 
C
C     Read vertex ID's and coordinates.
C
      FORMAT = 'I D D D'
 
      DO I = 1, NV
         NREC   = NREC + 1
         CALL RDFFDI ( INFILE, NREC, FORMAT, ND, NI, ARD, ARI, EOF )
         IF ( EOF ) THEN
            CALL EXTMSI ( 'End of file after line #.', '#', NREC )
         END IF
         VID(I) = ARI(1)
         X(I)   = ARD(1)
         Y(I)   = ARD(2)
         Z(I)   = ARD(3)
      END DO
 
C
C     Read number of plates.
C
      NREC   = NREC + 1
      FORMAT = 'I'
 
      CALL RDFFDI ( INFILE, NREC, FORMAT, ND, NI, ARD, ARI, EOF )
      IF ( EOF ) THEN
         CALL EXTMSI ( 'End of file after line #.', '#', NREC )
      END IF
 
      NP = ARI(1)
 
      IF ( NP .GT. MAXPLT ) THEN
         CALL SETMSG ( 'Number of plates exceeds limit.'          )
         CALL SIGERR ( 'SPICE(TOOMANYPLATES)'                     )
      END IF
 
C
C     Read each plate's vertex ID's.
C
      FORMAT = 'I I I I'
 
      DO I = 1, NP
         NREC   = NREC + 1
         CALL RDFFDI ( INFILE, NREC, FORMAT, ND, NI, ARD, ARI, EOF )
         IF ( EOF ) THEN
            CALL EXTMSI ( 'End of file after line #.', '#', NREC )
         END IF
         PID(I) = ARI(1)
         V1(I)  = ARI(2)
         V2(I)  = ARI(3)
         V3(I)  = ARI(4)
      END DO
C
C     CALl TOSTDO ( ' ' )
      CALL TOSTDO ( '...Done reading plate model flatfile.' )
      CALL TOSTDO ( ' ' )
 
      CALL CHKOUT ( 'RDFFPL' )
 
      RETURN
      END
